@@ -3,11 +3,11 @@ use super::*;
 /// A binding of `name` to `value`
 #[derive(Debug, Clone, PartialEq, Serialize)]
 pub(crate) struct Binding<'src, V = String> {
-  /// Export binding as an environment variable to child processes
   pub(crate) export: bool,
-  /// Binding name
+  #[serde(skip)]
+  pub(crate) file_depth: u32,
   pub(crate) name: Name<'src>,
-  /// Binding value
+  pub(crate) private: bool,
   pub(crate) value: V,
 }
 
