@@ -16,6 +16,7 @@ pub(crate) struct Config {
   pub(crate) dotenv_path: Option<PathBuf>,
   pub(crate) dry_run: bool,
   pub(crate) dump_format: DumpFormat,
+  pub(crate) explain: bool,
   pub(crate) highlight: bool,
   pub(crate) invocation_directory: PathBuf,
   pub(crate) list_heading: String,
@@ -24,6 +25,7 @@ pub(crate) struct Config {
   pub(crate) load_dotenv: bool,
   pub(crate) no_aliases: bool,
   pub(crate) no_dependencies: bool,
+  pub(crate) one: bool,
   pub(crate) search_config: SearchConfig,
   pub(crate) shell: Option<String>,
   pub(crate) shell_args: Option<Vec<String>>,
@@ -88,6 +90,7 @@ mod arg {
   pub(crate) const DOTENV_PATH: &str = "DOTENV-PATH";
   pub(crate) const DRY_RUN: &str = "DRY-RUN";
   pub(crate) const DUMP_FORMAT: &str = "DUMP-FORMAT";
+  pub(crate) const EXPLAIN: &str = "EXPLAIN";
   pub(crate) const GLOBAL_JUSTFILE: &str = "GLOBAL-JUSTFILE";
   pub(crate) const HIGHLIGHT: &str = "HIGHLIGHT";
   pub(crate) const JUSTFILE: &str = "JUSTFILE";
@@ -98,6 +101,7 @@ mod arg {
   pub(crate) const NO_DEPS: &str = "NO-DEPS";
   pub(crate) const NO_DOTENV: &str = "NO-DOTENV";
   pub(crate) const NO_HIGHLIGHT: &str = "NO-HIGHLIGHT";
+  pub(crate) const ONE: &str = "ONE";
   pub(crate) const QUIET: &str = "QUIET";
   pub(crate) const SET: &str = "SET";
   pub(crate) const SHELL: &str = "SHELL";
@@ -206,6 +210,13 @@ impl Config {
           .value_name("FORMAT")
           .help("Dump justfile as <FORMAT>"),
       )
+      .arg(
+        Arg::new(arg::EXPLAIN)
+          .action(ArgAction::SetTrue)
+          .long("explain")
+          .env("JUST_EXPLAIN")
+          .help("Print recipe doc comment before running it"),
+      )
       .arg(
         Arg::new(arg::GLOBAL_JUSTFILE)
           .action(ArgAction::SetTrue)
@@ -288,6 +299,13 @@ impl Config {
           .help("Don't highlight echoed recipe lines in bold")
           .overrides_with(arg::HIGHLIGHT),
       )
+      .arg(
+        Arg::new(arg::ONE)
+          .long("one")
+          .env("JUST_ONE")
+          .action(ArgAction::SetTrue)
+          .help("Forbid multiple recipes from being invoked on the command line"),
+      )
       .arg(
         Arg::new(arg::QUIET)
           .short('q')
@@ -685,6 +703,7 @@ impl Config {
     };
 
     let unstable = matches.get_flag(arg::UNSTABLE) || subcommand == Subcommand::Summary;
+    let explain = matches.get_flag(arg::EXPLAIN);
 
     Ok(Self {
       check: matches.get_flag(arg::CHECK),
@@ -702,6 +721,7 @@ impl Config {
         .get_one::<DumpFormat>(arg::DUMP_FORMAT)
         .unwrap()
         .clone(),
+      explain,
       highlight: !matches.get_flag(arg::NO_HIGHLIGHT),
       invocation_directory: env::current_dir().context(config_error::CurrentDirContext)?,
       list_heading: matches.get_one::<String>(arg::LIST_HEADING).unwrap().into(),
@@ -710,6 +730,7 @@ impl Config {
       load_dotenv: !matches.get_flag(arg::NO_DOTENV),
       no_aliases: matches.get_flag(arg::NO_ALIASES),
       no_dependencies: matches.get_flag(arg::NO_DEPS),
+      one: matches.get_flag(arg::ONE),
       search_config,
       shell: matches.get_one::<String>(arg::SHELL).map(Into::into),
       shell_args: if matches.get_flag(arg::CLEAR_SHELL_ARGS) {
