@@ -3,7 +3,7 @@ use super::*;
 fn case(justfile: &str, value: Value) {
   Test::new()
     .justfile(justfile)
-    .args(["--dump", "--dump-format", "json", "--unstable"])
+    .args(["--dump", "--dump-format", "json"])
     .stdout(format!("{}\n", serde_json::to_string(&value).unwrap()))
     .run();
 }
@@ -18,6 +18,7 @@ fn alias() {
     ",
     json!({
       "first": "foo",
+      "doc": null,
       "aliases": {
         "f": {
           "name": "f",
@@ -26,6 +27,7 @@ fn alias() {
         }
       },
       "assignments": {},
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -34,6 +36,7 @@ fn alias() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -43,18 +46,25 @@ fn alias() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "ignore_comments": false,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -71,25 +81,88 @@ fn assignment() {
           "export": false,
           "name": "foo",
           "value": "bar",
+          "private": false,
         }
       },
       "first": null,
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {},
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
+      "warnings": [],
+    }),
+  );
+}
+
+#[test]
+fn private_assignment() {
+  case(
+    "
+      _foo := 'foo'
+      [private]
+      bar := 'bar'
+    ",
+    json!({
+      "aliases": {},
+      "assignments": {
+        "_foo": {
+          "export": false,
+          "name": "_foo",
+          "value": "foo",
+          "private": true,
+        },
+        "bar": {
+          "export": false,
+          "name": "bar",
+          "value": "bar",
+          "private": true,
+        },
+      },
+      "first": null,
+      "doc": null,
+      "groups": [],
+      "modules": {},
+      "recipes": {},
+      "settings": {
+        "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
+        "dotenv_filename": null,
+        "dotenv_load": false,
+        "dotenv_path": null,
+        "dotenv_required": false,
+        "export": false,
+        "fallback": false,
+        "ignore_comments": false,
+        "positional_arguments": false,
+        "quiet": false,
+        "shell": null,
+        "tempdir" : null,
+        "unstable": false,
+        "windows_powershell": false,
+        "windows_shell": null,
+        "working_directory" : null,
+      },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -107,6 +180,8 @@ fn body() {
       "aliases": {},
       "assignments": {},
       "first": "foo",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -118,6 +193,7 @@ fn body() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -127,18 +203,24 @@ fn body() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -155,12 +237,15 @@ fn dependencies() {
       "aliases": {},
       "assignments": {},
       "first": "foo",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "bar": {
           "attributes": [],
           "doc": null,
           "name": "bar",
+          "namepath": "bar",
           "body": [],
           "dependencies": [{
             "arguments": [],
@@ -177,6 +262,7 @@ fn dependencies() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -187,18 +273,24 @@ fn dependencies() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -227,18 +319,22 @@ fn dependency_argument() {
     json!({
       "aliases": {},
       "first": "foo",
+      "doc": null,
       "assignments": {
         "x": {
           "export": false,
           "name": "x",
           "value": "foo",
+          "private": false,
         },
       },
+      "groups": [],
       "modules": {},
       "recipes": {
         "bar": {
           "doc": null,
           "name": "bar",
+          "namepath": "bar",
           "body": [],
           "dependencies": [{
             "arguments": [
@@ -247,7 +343,7 @@ fn dependency_argument() {
               ["concatenate", "a", "b"],
               ["evaluate", "echo"],
               ["variable", "x"],
-              ["if", "==", "a", "b", "c", "d"],
+              ["if", ["==", "a", "b"], "c", "d"],
               ["call", "arch"],
               ["call", "env_var", "foo"],
               ["call", "join", "a", "b"],
@@ -267,6 +363,7 @@ fn dependency_argument() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [
             {
               "name": "args",
@@ -284,18 +381,24 @@ fn dependency_argument() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -313,6 +416,7 @@ fn duplicate_recipes() {
     ",
     json!({
       "first": "foo",
+      "doc": null,
       "aliases": {
         "f": {
           "attributes": [],
@@ -321,6 +425,7 @@ fn duplicate_recipes() {
         }
       },
       "assignments": {},
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -328,6 +433,7 @@ fn duplicate_recipes() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [
             {
               "name": "bar",
@@ -345,18 +451,72 @@ fn duplicate_recipes() {
       },
       "settings": {
         "allow_duplicate_recipes": true,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
+      "warnings": [],
+    }),
+  );
+}
+
+#[test]
+fn duplicate_variables() {
+  case(
+    "
+      set allow-duplicate-variables
+      x := 'foo'
+      x := 'bar'
+    ",
+    json!({
+      "aliases": {},
+      "assignments": {
+        "x": {
+          "export": false,
+          "name": "x",
+          "value": "bar",
+          "private": false,
+        }
+      },
+      "first": null,
+      "doc": null,
+      "groups": [],
+      "modules": {},
+      "recipes": {},
+      "settings": {
+        "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": true,
+        "dotenv_filename": null,
+        "dotenv_load": false,
+        "dotenv_path": null,
+        "dotenv_required": false,
+        "export": false,
+        "fallback": false,
+        "ignore_comments": false,
+        "positional_arguments": false,
+        "quiet": false,
+        "shell": null,
+        "tempdir" : null,
+        "unstable": false,
+        "windows_powershell": false,
+        "windows_shell": null,
+        "working_directory" : null,
+      },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -369,7 +529,9 @@ fn doc_comment() {
     json!({
       "aliases": {},
       "first": "foo",
+      "doc": null,
       "assignments": {},
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -377,6 +539,7 @@ fn doc_comment() {
           "dependencies": [],
           "doc": "hello",
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -387,18 +550,24 @@ fn doc_comment() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -412,22 +581,30 @@ fn empty_justfile() {
       "aliases": {},
       "assignments": {},
       "first": null,
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {},
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -447,7 +624,9 @@ fn parameters() {
     json!({
       "aliases": {},
       "first": "a",
+      "doc": null,
       "assignments": {},
+      "groups": [],
       "modules": {},
       "recipes": {
         "a": {
@@ -456,6 +635,7 @@ fn parameters() {
           "dependencies": [],
           "doc": null,
           "name": "a",
+          "namepath": "a",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -467,6 +647,7 @@ fn parameters() {
           "dependencies": [],
           "doc": null,
           "name": "b",
+          "namepath": "b",
           "parameters": [
             {
               "name": "x",
@@ -486,6 +667,7 @@ fn parameters() {
           "dependencies": [],
           "doc": null,
           "name": "c",
+          "namepath": "c",
           "parameters": [
             {
               "name": "x",
@@ -505,6 +687,7 @@ fn parameters() {
           "dependencies": [],
           "doc": null,
           "name": "d",
+          "namepath": "d",
           "parameters": [
             {
               "name": "x",
@@ -524,6 +707,7 @@ fn parameters() {
           "dependencies": [],
           "doc": null,
           "name": "e",
+          "namepath": "e",
           "parameters": [
             {
               "name": "x",
@@ -543,6 +727,7 @@ fn parameters() {
           "dependencies": [],
           "doc": null,
           "name": "f",
+          "namepath": "f",
           "parameters": [
             {
               "name": "x",
@@ -560,18 +745,24 @@ fn parameters() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -589,6 +780,8 @@ fn priors() {
       "aliases": {},
       "assignments": {},
       "first": "a",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "a": {
@@ -596,6 +789,7 @@ fn priors() {
           "dependencies": [],
           "doc": null,
           "name": "a",
+          "namepath": "a",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -617,6 +811,7 @@ fn priors() {
           ],
           "doc": null,
           "name": "b",
+          "namepath": "b",
           "private": false,
           "quiet": false,
           "shebang": false,
@@ -629,6 +824,7 @@ fn priors() {
           "dependencies": [],
           "doc": null,
           "name": "c",
+          "namepath": "c",
           "parameters": [],
           "private": false,
           "quiet": false,
@@ -640,18 +836,24 @@ fn priors() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -665,6 +867,8 @@ fn private() {
       "aliases": {},
       "assignments": {},
       "first": "_foo",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "_foo": {
@@ -672,6 +876,7 @@ fn private() {
           "dependencies": [],
           "doc": null,
           "name": "_foo",
+          "namepath": "_foo",
           "parameters": [],
           "priors": 0,
           "private": true,
@@ -682,18 +887,24 @@ fn private() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -707,6 +918,8 @@ fn quiet() {
       "aliases": {},
       "assignments": {},
       "first": "foo",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -714,6 +927,7 @@ fn quiet() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -724,18 +938,24 @@ fn quiet() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -751,6 +971,7 @@ fn settings() {
       set export
       set fallback
       set positional-arguments
+      set quiet
       set ignore-comments
       set shell := ['a', 'b', 'c']
       foo:
@@ -760,6 +981,8 @@ fn settings() {
       "aliases": {},
       "assignments": {},
       "first": "foo",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -767,6 +990,7 @@ fn settings() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -777,21 +1001,27 @@ fn settings() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": "filename",
         "dotenv_load": true,
         "dotenv_path": "path",
+        "dotenv_required": false,
         "export": true,
         "fallback": true,
         "ignore_comments": true,
         "positional_arguments": true,
+        "quiet": true,
         "shell": {
           "arguments": ["b", "c"],
           "command": "a",
         },
         "tempdir": null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -808,6 +1038,8 @@ fn shebang() {
       "aliases": {},
       "assignments": {},
       "first": "foo",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -815,6 +1047,7 @@ fn shebang() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -825,18 +1058,24 @@ fn shebang() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir": null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -850,6 +1089,8 @@ fn simple() {
       "aliases": {},
       "assignments": {},
       "first": "foo",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -857,6 +1098,7 @@ fn simple() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -867,18 +1109,24 @@ fn simple() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "ignore_comments": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir": null,
+        "unstable": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -895,6 +1143,8 @@ fn attribute() {
       "aliases": {},
       "assignments": {},
       "first": "foo",
+      "doc": null,
+      "groups": [],
       "modules": {},
       "recipes": {
         "foo": {
@@ -903,6 +1153,7 @@ fn attribute() {
           "dependencies": [],
           "doc": null,
           "name": "foo",
+          "namepath": "foo",
           "parameters": [],
           "priors": 0,
           "private": false,
@@ -912,18 +1163,24 @@ fn attribute() {
       },
       "settings": {
         "allow_duplicate_recipes": false,
+        "allow_duplicate_variables": false,
         "dotenv_filename": null,
-        "dotenv_load": null,
+        "dotenv_load": false,
         "dotenv_path": null,
+        "dotenv_required": false,
         "export": false,
         "fallback": false,
         "positional_arguments": false,
+        "quiet": false,
         "shell": null,
         "tempdir" : null,
+        "unstable": false,
         "ignore_comments": false,
         "windows_powershell": false,
         "windows_shell": null,
+        "working_directory" : null,
       },
+      "unexports": [],
       "warnings": [],
     }),
   );
@@ -934,25 +1191,29 @@ fn module() {
   Test::new()
     .justfile(
       "
+      # hello
       mod foo
     ",
     )
     .tree(tree! {
       "foo.just": "bar:",
     })
-    .args(["--dump", "--dump-format", "json", "--unstable"])
-    .test_round_trip(false)
+    .args(["--dump", "--dump-format", "json"])
     .stdout(format!(
       "{}\n",
       serde_json::to_string(&json!({
         "aliases": {},
         "assignments": {},
         "first": null,
+        "doc": null,
+        "groups": [],
         "modules": {
           "foo": {
             "aliases": {},
             "assignments": {},
             "first": "bar",
+            "doc": "hello",
+            "groups": [],
             "modules": {},
             "recipes": {
               "bar": {
@@ -961,6 +1222,7 @@ fn module() {
                 "dependencies": [],
                 "doc": null,
                 "name": "bar",
+                "namepath": "foo::bar",
                 "parameters": [],
                 "priors": 0,
                 "private": false,
@@ -970,36 +1232,143 @@ fn module() {
             },
             "settings": {
               "allow_duplicate_recipes": false,
+              "allow_duplicate_variables": false,
               "dotenv_filename": null,
-              "dotenv_load": null,
+              "dotenv_load": false,
               "dotenv_path": null,
+              "dotenv_required": false,
               "export": false,
               "fallback": false,
               "positional_arguments": false,
+              "quiet": false,
               "shell": null,
               "tempdir" : null,
+              "unstable": false,
               "ignore_comments": false,
               "windows_powershell": false,
               "windows_shell": null,
+              "working_directory" : null,
             },
+            "unexports": [],
             "warnings": [],
           },
         },
         "recipes": {},
         "settings": {
           "allow_duplicate_recipes": false,
+          "allow_duplicate_variables": false,
           "dotenv_filename": null,
-          "dotenv_load": null,
+          "dotenv_load": false,
           "dotenv_path": null,
+          "dotenv_required": false,
           "export": false,
           "fallback": false,
           "positional_arguments": false,
+          "quiet": false,
           "shell": null,
           "tempdir" : null,
+          "unstable": false,
           "ignore_comments": false,
           "windows_powershell": false,
           "windows_shell": null,
+          "working_directory" : null,
         },
+        "unexports": [],
+        "warnings": [],
+      }))
+      .unwrap()
+    ))
+    .run();
+}
+
+#[test]
+fn module_group() {
+  Test::new()
+    .justfile(
+      "
+      [group('alpha')]
+      mod foo
+    ",
+    )
+    .tree(tree! {
+      "foo.just": "bar:",
+    })
+    .args(["--dump", "--dump-format", "json"])
+    .stdout(format!(
+      "{}\n",
+      serde_json::to_string(&json!({
+        "aliases": {},
+        "assignments": {},
+        "first": null,
+        "doc": null,
+        "groups": [],
+        "modules": {
+          "foo": {
+            "aliases": {},
+            "assignments": {},
+            "first": "bar",
+            "doc": null,
+            "groups": ["alpha"],
+            "modules": {},
+            "recipes": {
+              "bar": {
+                "attributes": [],
+                "body": [],
+                "dependencies": [],
+                "doc": null,
+                "name": "bar",
+                "namepath": "foo::bar",
+                "parameters": [],
+                "priors": 0,
+                "private": false,
+                "quiet": false,
+                "shebang": false,
+              }
+            },
+            "settings": {
+              "allow_duplicate_recipes": false,
+              "allow_duplicate_variables": false,
+              "dotenv_filename": null,
+              "dotenv_load": false,
+              "dotenv_path": null,
+              "dotenv_required": false,
+              "export": false,
+              "fallback": false,
+              "positional_arguments": false,
+              "quiet": false,
+              "shell": null,
+              "tempdir" : null,
+              "unstable": false,
+              "ignore_comments": false,
+              "windows_powershell": false,
+              "windows_shell": null,
+              "working_directory" : null,
+            },
+            "unexports": [],
+            "warnings": [],
+          },
+        },
+        "recipes": {},
+        "settings": {
+          "allow_duplicate_recipes": false,
+          "allow_duplicate_variables": false,
+          "dotenv_filename": null,
+          "dotenv_load": false,
+          "dotenv_path": null,
+          "dotenv_required": false,
+          "export": false,
+          "fallback": false,
+          "positional_arguments": false,
+          "quiet": false,
+          "shell": null,
+          "tempdir" : null,
+          "unstable": false,
+          "ignore_comments": false,
+          "windows_powershell": false,
+          "windows_shell": null,
+          "working_directory" : null,
+        },
+        "unexports": [],
         "warnings": [],
       }))
       .unwrap()
