@@ -3,9 +3,8 @@ pub(crate) use {
     assert_stdout::assert_stdout,
     assert_success::assert_success,
     tempdir::tempdir,
-    test::{Output, Test},
+    test::{assert_eval_eq, Output, Test},
   },
-  cradle::input::Input,
   executable_path::executable_path,
   just::unindent,
   libc::{EXIT_FAILURE, EXIT_SUCCESS},
@@ -20,7 +19,7 @@ pub(crate) use {
     fs,
     io::Write,
     iter,
-    path::{Path, PathBuf, MAIN_SEPARATOR},
+    path::{Path, PathBuf, MAIN_SEPARATOR, MAIN_SEPARATOR_STR},
     process::{Command, Stdio},
     str,
   },
@@ -33,9 +32,12 @@ pub(crate) use {
 mod test;
 
 mod allow_duplicate_recipes;
+mod allow_duplicate_variables;
 mod assert_stdout;
 mod assert_success;
+mod assertions;
 mod attributes;
+mod backticks;
 mod byte_order_mark;
 mod changelog;
 mod choose;
@@ -43,17 +45,24 @@ mod command;
 mod completions;
 mod conditional;
 mod confirm;
+mod constants;
+mod datetime;
 mod delimiters;
+mod directories;
 mod dotenv;
 mod edit;
 mod equals;
 mod error_messages;
 mod evaluate;
 mod examples;
+mod explain;
 mod export;
 mod fallback;
 mod fmt;
 mod functions;
+#[cfg(unix)]
+mod global;
+mod groups;
 mod ignore_comments;
 mod imports;
 mod init;
@@ -62,13 +71,18 @@ mod interrupts;
 mod invocation_directory;
 mod json;
 mod line_prefixes;
+mod list;
+mod man;
 mod misc;
 mod modules;
 mod multibyte_char;
 mod newline_escape;
+mod no_aliases;
 mod no_cd;
+mod no_dependencies;
 mod no_exit_message;
 mod os_attributes;
+mod parameters;
 mod parser;
 mod positional_arguments;
 mod private;
@@ -78,19 +92,25 @@ mod readme;
 mod recursion_limit;
 mod regexes;
 mod run;
+mod script;
 mod search;
 mod search_arguments;
 mod shadowing_parameters;
 mod shebang;
 mod shell;
+mod shell_expansion;
 mod show;
 mod slash_operator;
 mod string;
 mod subsequents;
 mod summary;
 mod tempdir;
+mod timestamps;
 mod undefined_variables;
+mod unexport;
 mod unstable;
+#[cfg(windows)]
+mod windows;
 #[cfg(target_family = "windows")]
 mod windows_shell;
 mod working_directory;
